"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const build_kit_1 = require("@genesislcap/build-kit");
const consola_1 = tslib_1.__importDefault(require("consola"));
const rollup_1 = require("rollup");
const options_1 = require("./options");
exports.default = (ctx) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const { dirs: { cwd }, cli: { isDev, isAnalyze, isBuild, options }, env: { isCI }, config: { externals, clean, copyAssets, entry }, pkg: { nameUnscoped }, } = ctx;
    if (clean) {
        yield (0, build_kit_1.rm)(['dist', '.rollup.cache']);
    }
    if (Array.isArray(copyAssets)) {
        const copyfiles = yield (0, build_kit_1.resolveBin)('copyfiles');
        copyAssets.forEach((glob) => {
            (0, build_kit_1.run)(cwd, `${copyfiles} --verbose --up 1 src/${glob} ./dist/esm`);
        });
    }
    const start = Date.now();
    const openStats = options.open && !isCI;
    const input = (0, options_1.getInputOptions)(externals, isAnalyze, openStats, entry);
    const output = (0, options_1.getOutputOptions)(externals, isAnalyze, isDev, nameUnscoped);
    const files = output.map((o) => o.file);
    consola_1.default.log((0, build_kit_1.white)(`\n${(0, build_kit_1.bold)(input.input)} → ${(0, build_kit_1.bold)(files.join(', '))}...`));
    if (isDev) {
        const watchOptions = Object.assign(Object.assign({}, input), { output });
        const watcher = (0, rollup_1.watch)(watchOptions);
        watcher.on('event', (event) => {
            if (event.code === 'BUNDLE_END') {
                event.result.close();
            }
        });
    }
    else if (isAnalyze || isBuild) {
        const bundle = yield (0, rollup_1.rollup)(input);
        yield Promise.all(output.map(bundle.write));
        yield bundle.close();
        consola_1.default.success((0, build_kit_1.green)(`bundled ${(0, build_kit_1.bold)(bundle.watchFiles.length)} files → ${(0, build_kit_1.bold)(files.join(', '))} in ${(0, build_kit_1.bold)((0, build_kit_1.ms)(Date.now() - start))}`));
    }
    else {
        throw new Error(`Unrecognized command: ${JSON.stringify(options)}`);
    }
});
